@@ -80,7 +80,6 @@ var menudata={children:[
 {text:"f",url:"globals_func.html#index_f"}]},
 {text:"Typedefs",url:"globals_type.html"},
 {text:"Enumerations",url:"globals_enum.html"},
-{text:"Enumerator",url:"globals_eval.html",children:[
-{text:"f",url:"globals_eval.html#index_f"}]},
+{text:"Enumerator",url:"globals_eval.html"},
 {text:"Macros",url:"globals_defs.html",children:[
 {text:"f",url:"globals_defs.html#index_f"}]}]}]}]}
